# MogHA

本项目为MogDB/openGuass数据库的高可用组件，用于管理数据库的高可用切换等处理。

注：前置警告，不建议使用gs_om管理数据库的主备实例启停，建议使用gs_ctl 命令启停数据库服务，已知gs_om存在部分操作无法跟随更新的问题

```bash
gs_ctl start -M primary # 启动为主库
gs_ctl start -M standby # 启动为备库
gs_ctl build -b full # 如果备库出现错误, 需要重置时候 (需要删除当前备库所有数据,操作前做好备份)
```

## 安装

安装包下载地址: https://docs.mogdb.io/zh/mogha/v2.3/release-notes

下载后得到一个类似 `mogha-x.x.x.tar.gz` 的压缩包（`x` 代表版本号）

### 前期准备

#### omm 用户 gsql, gs_ctl 免密执行

测试方式，执行下面的命令不需要输入密码即可查询到集群状态：

```
gs_ctl -D [PGDATA] query 
```

## 安装步骤

> 安装请使用 root 操作

将压缩包拷贝到要部署HA的服务器上，存放目录建议选择`数据库安装用户`的家目录，以 `omm` 为例，则存放目录为 `/home/omm/`，**为了便于演示，后面不再声明，默认使用该目录进行演示**

### 解压缩安装包

```bash
cd /home/omm
tar -zxf mogha-2.3.0.tar.gz
```

解压后会在当前目录下得到一个 `mogha` 的文件夹

### 安装HA服务

安装脚本执行方式：`sudo ./install.sh USER PGDATA`

`USER` 是指数据库安装用户，在本例中指 `omm` 用户

`PGDATA` 是指数据库数据目录，本例子假设数据目录为 `/opt/mogdb/data`

```bash
# 进入刚刚解压后的文件夹
cd /home/omm/mogha
# 执行安装脚本
sudo ./install.sh omm /opt/mogdb/data

# 安装过程，输出类似一下内容
[2021-11-13 23:47:52]: MogHA 安装目录为：/home/omm/mogha
[2021-11-13 23:47:52]: 数据库安装用户为：omm, 所在用户组为：dbgrp
[2021-11-13 23:47:52]: 数据库数据目录为：/opt/mogdb/data
[2021-11-13 23:47:52]: LD_LIBRARY_PATH=/opt/mogdb/app/lib:/opt/mogdb/tool/lib:
[2021-11-13 23:47:52]: GAUSSHOME=/opt/mogdb/app
[2021-11-13 23:47:52]: 数据库端口: 26000
[2021-11-13 23:47:52]: 当前系统架构为：aarch64
[2021-11-13 23:47:52]: 修改安装目录及其子目录或文件的用户和用户组...
[2021-11-13 23:47:52]: 生成 mogha.service 文件...
[2021-11-13 23:47:52]: 复制 mogha.service 文件到 /usr/lib/systemd/system/
[2021-11-13 23:47:52]: 重载配置文件
[2021-11-13 23:47:52]: mogha 服务注册成功
[2021-11-13 23:47:52]: 配置 omm 用户的 sudo 权限到 /etc/sudoers 文件
[2021-11-13 23:47:52]: 配置权限成功：NOPASSWD:/usr/bin/systemctl,NOPASSWD:/usr/sbin/ifconfig
[2021-11-13 23:47:52]: 发现 node.conf 已经存在，不再自动生成
[2021-11-13 23:47:52]: MogHA 安装成功!

请优先修改 /home/omm/mogha/node.conf 配置文件后再启动服务 !!!

管理 MogHA 服务可以通过 systemctl 工具:
    启动：sudo systemctl start mogha
    停止：sudo systemctl stop mogha
    重启：sudo systemctl restart mogha

卸载 MogHA 服务：
    sudo ./uninstall.sh mogha
```

**需要注意的是**：

执行完安装脚本后，不要立即启动服务，HA的运行强烈依赖安装目录中 `node.conf` 的配置，我们需要先正确的配置了这个文件再启动服务。当第一次安装的时候，`node.conf`
一般是没有的，脚本会根据 `node.conf.tmpl` 模板文件自动生成一份 `node.conf`， 所以可以看到执行的输出最后重点强调了: `请根据您的集群信息修改 node.conf 中的配置后再启动服务`
，后面将阐述，我们应该如何来进行配置 `node.conf` 。

### node.conf 的配置

#### 必须用户确认配置的字段

- 数据库连接端口

```bash
# 数据库端口，用于连接数据库
db_port=26000
```

- HA通信端口

```bash
# ha节点之间心跳端口，如果有防火墙，需要打开这个端口的访问
agent_port=8081
```

- host 主机列表（仅选取 host1 做演示，其他相同）

  主机列表需要用户将自己的数据库集群中的所有主备实例所在的服务器的业务IP填写到 `ip` 字段。

  如果没有心跳IP的话，可以不填写置空，如果有则填写，如果有多个心跳IP，则以英文半角逗号分隔填写

```bash
# 这部分从host1-8，每个代表一个机器
[host1]
# 业务网络ip
ip=192.168.1.111
# 心跳网络ip，允许配置多个心跳网络，以逗号隔开
heartbeat_ips=192.168.1.112,192.168.1.113

```

- zone 机房列表（仅选取 zone1 做演示，其他相同）

  zone 在这里代表的是机房的概念

  `vip`：不同的机房拥有各自的虚拟IP（VIP）,虚拟IP用于业务访问，解耦服务器的私有IP，便于主备切换

  `arping`： 可以留空，系统会采用广播地址

  `ping_list`：用于检查网络的中间节点IP，一般可以选择网关地址

  `hosts`：本机房中的主机列表，特别强调，这里**不是**填写主机名或主机IP，这里是用于**归类**上一步中填写的 host 列表，所以这个字段应该填写的是本配置文件中定义的 `host1`，`host2` ... 等等这种
  section 名，举例：假如 `host1`, `host2` 这两个机器同属于 `zone1` 这个机房，那么这里的 `hosts` 应该填写为： `hosts=host1,host2` （以英文半角逗号分隔）

  `cascade`： 本机房中的级联备列表，配置方式同 `hosts`

```bash
# 这部分主要用于区分机房，不同机房会有自己的VIP，
[zone1]
# 本机房的VIP
vip=
# 本机房的arping地址，如果不知道，可以留空，会采用arp广播模式
arping=
# 检查网络的中间节点，一般是可以ping通的网关地址
ping_list=
# 本机房的机器列表，例如 host1,host2,host3
hosts=
# 本机房的级联备库列表
cascade=
```

#### 其他可选项配置

```bash
[config]
# 心跳间隔时间，也是主流程检查的间隔时间，一般设置为3-5s
heartbeat_interval=3

# 主库丢失的探测时间，当主库宕机无法ping到，认为持续到这个时长之后，备库可以切换为主库
primary_lost_timeout=10

# 主库的孤单时间，当主库无法连接到其他机器，ping不到其他任何机器，认为主库网络出现问题
# 当持续超过这个时长，认为主库实例需要关闭(mogdb中会设置为readonly)
primary_lonely_timeout=10

# 当双主状态持续指定时长之后，认为需要做双主判断，
# 因为在维护性操作中，会有可能存在极短时间的双主状态（此时事实上数据库不允许写入）
double_primary_timeout=10

# 数据库的操作系统用户，通常为omm或者postgres
db_user=__USER__

# 数据库的数据目录，用于信号发送等本地通讯行为
db_datadir=__PGDATA__

# 本地元数据文件类型:
#   - "json" : json 格式
#   - "bin" : 二进制格式 (如果该参数没有配置，默认使用 bin)
meta_file_type=json

# 本地元数据存储位置，自动维护无需人工干预，当元数据库不可用时候，用这个数据作为主库判断
primary_info=__INSTALLDIR__/primary_info

# 本地元数据存储位置，自动维护无需人工干预，当主库丢失，备库依赖这个去判断自身同步状态
standby_info=__INSTALLDIR__/standby_info

# 是否为数据库实例设置cpu限制，True的话，会保留至少一个cpu出来不分配给数据库用
taskset=True

# 是否使用 lite 模式，可选值：True / False
# lite 模式下，即便是一主多备，也只需在主和同步备启动服务。
# 该值为 False 的话，即为 full 模式，需要在所有节点启动高可用服务。
# 二者的区别：lite 模式，HA服务不会修改数据库的主备相关配置，而 full 模式下会自动修改
lite_mode=False

# 是否使用元数据库。
# 如果值为 True，需在下面的 [meta] 部分，配置元数据库连接参数，
# 如果值为 False，[meta] 部分可以置空
use_meta=False

# 设置输出的日志格式
logger_format=%(asctime)s - %(name)s - %(levelname)s [%(filename)s:%(lineno)d]: %(message)s

# 设置除了主备相关的机器，允许可以访问到web接口的IP列表, 多个IP时英文半角逗号分隔
allow_ips=

# [v2.1新增] 主库实例进程未启动时，如何处理
# 支持两种处理方式：
#   restart: 尝试重启，尝试次数在 restart_instance_limit 参数中设定
#   failover: 备库升为主，主库尝试重建数据库后启动为备
primary_down_handle_method=failover

# [v2.1新增] 重启实例最大尝试条件: times/minutes
# 例如： 10/3 最多尝试10次或者3分钟，任何一个条件先满足就不再尝试。
restart_strategy=10/3
```

### 启动服务

确认配置完成 `node.conf` 以后，就可以启动 HA 服务了，启动方式为：

```bash
sudo systemctl start mogha
```

启动后检查 mogha 的运行状态:

```bash
$ ps -ef | grep mogha
omm      3401876       1  3 23:49 ?        00:00:00 /home/omm/mogha/mogha-2.2.3-dev-aarch64 -c /home/omm/mogha/node.conf
omm      3401889 3401876  4 23:49 ?        00:00:00 mogha: watchdog
omm      3401950 3401889  0 23:49 ?        00:00:00 mogha: http-server
omm      3401951 3401889  0 23:49 ?        00:00:00 mogha: heartbeat
```

重启HA:

```bash
sudo systemctl restart mogha
```

停止HA：

```bash
sudo systemctl stop mogha
```

### 日志文件

HA内部会生成两个日志文件：

- mogha_heartbeat.log

  这个日志文件会存放在安装目录中，在本示例中为 `/home/omm/mogha`，这个日志主要记录HA的心跳日志，也是排查问题主要的优先检查的日志文件。

- mogha_web.log

  这个日志文件存放目录和心跳日志相同。这个日志主要记录 web api 接口的请求日志。

## 卸载软件

进入安装目录，执行卸载脚本即可：

```bash
cd /home/omm/mogha
sudo ./uninstall.sh
```
